@@ -46,6 +46,7 @@ def collect(args):
     report['service_job_id'] = args.service_job_id
 
     report['source_files'] = []
+    abs_root = os.path.abspath(args.root)
     for root, dirs, files in os.walk(args.root):
         filtered_dirs = []
         for dirpath in dirs:
@@ -55,40 +56,46 @@ def collect(args):
         dirs[:] = filtered_dirs
 
         for filepath in files:
-            if is_source_file(filepath):
-                src_path = os.path.relpath(os.path.join(root, filepath),
-                                           args.root)
-                gcov_path = src_path + '.gcov'
-                src_report = {}
-                src_report['name'] = src_path
-                with open(src_path) as fobj:
-                    src_report['source'] = fobj.read()
+            if os.path.splitext(filepath)[1] == '.gcov':
+                gcov_path = os.path.join(os.path.join(root, filepath))
+                with open(gcov_path) as fobj:
+                    source_file_line = fobj.readline()
+                    source_file_path = source_file_line.split(':')[-1].strip()
+                    if not os.path.isabs(source_file_path):
+                        source_file_path = os.path.abspath(
+                            os.path.join(root, source_file_path))
+                    src_path = os.path.relpath(source_file_path, abs_root)
+                    if len(src_path) > 3 and src_path[:3] == '../':
+                        continue
 
-                coverage = []
-                if os.path.exists(gcov_path):
-                    with open(gcov_path) as fobj:
-                        for line in fobj:
-                            report_fields = line.split(':')
-                            cov_num = report_fields[0].strip()
-                            line_num = int(report_fields[1].strip())
-                            text = report_fields[2]
-                            if line_num == 0:
-                                continue
-                            if cov_num == '-':
+                    src_report = {}
+                    src_report['name'] = src_path
+                    with open(src_path) as src_file:
+                        src_report['source'] = src_file.read()
+
+                    coverage = []
+                    for line in fobj:
+                        report_fields = line.split(':')
+                        cov_num = report_fields[0].strip()
+                        line_num = int(report_fields[1].strip())
+                        text = report_fields[2]
+                        if line_num == 0:
+                            continue
+                        if cov_num == '-':
+                            coverage.append(None)
+                        elif cov_num == '#####':
+                            # Avoid false positives.
+                            if (text.lstrip().startswith('static') or
+                                    text.strip() == '}'):
                                 coverage.append(None)
-                            elif cov_num == '#####':
-                                # Avoid false positives.
-                                if (text.lstrip().startswith('static') or
-                                        text.strip() == '}'):
-                                    coverage.append(None)
-                                else:
-                                    coverage.append(0)
-                            elif cov_num == '=====':
-                                # This is indicitive of a gcov output parse
-                                # error.
-                                coverage.append(0)
                             else:
-                                coverage.append(int(cov_num))
+                                coverage.append(0)
+                        elif cov_num == '=====':
+                            # This is indicitive of a gcov output parse
+                            # error.
+                            coverage.append(0)
+                        else:
+                            coverage.append(int(cov_num))
                 src_report['coverage'] = coverage
                 report['source_files'].append(src_report)
     report['git'] = gitrepo.gitrepo('.')
